import React from 'react';
import PropTypes from 'prop-types';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableContainer from '@material-ui/core/TableContainer';
import TablePagination from '@material-ui/core/TablePagination';
import TableRow from '@material-ui/core/TableRow';
import { makeStyles } from '@material-ui/core/styles';
import { CustomTableHead } from "./CustomTableHead";

const useStyles = makeStyles({
  container: {
    maxHeight: 640,
  },
});

export function CustomTable(props) {
  const {columns, rows, handleRowClick} = props;
  const classes = useStyles();
  const [page, setPage] = React.useState(0);
  const [rowsPerPage, setRowsPerPage] = React.useState(10);
  const [order, setOrder] = React.useState('asc');
  const [orderBy, setOrderBy] = React.useState(null);

  const handleRequestSort = (property) => {
    const isAsc = orderBy === property && order === 'asc';
    setOrder(isAsc ? 'desc' : 'asc');
    setOrderBy(property);
  };


  const handleChangePage = (event, newPage) => {
    setPage(newPage);
  };

  const handleChangeRowsPerPage = (event) => {
    setRowsPerPage(+event.target.value);
    setPage(0);
  };

  const handleClick = (event, id) => {
    handleRowClick(id);
  };

  function descendingComparator(a, b, orderBy) {
    if (orderBy != null) {
      if (b[orderBy].value < a[orderBy].value) {
          return -1;
      }
      if (b[orderBy].value > a[orderBy].value) {
          return 1;
      }
    }
    return 0;
  }

  function getComparator(order, orderBy, customDescendingComparator) {
    return order === 'desc'
        ? (a, b) => customDescendingComparator ? customDescendingComparator(a, b) : descendingComparator(a, b, orderBy)
        : (a, b) => customDescendingComparator ? -customDescendingComparator(a, b) : -descendingComparator(a, b, orderBy);
    }

  function stableSort(array, comparator) {
    const stabilizedThis = array.map((el, index) => [el, index]);
    stabilizedThis.sort((a, b) => {
        const order = comparator(a[0], b[0]);
        if (order !== 0) return order;
        return a[1] - b[1];
    });
    return stabilizedThis.map((el) => el[0]);
  }

  return (
    <div>
      <TableContainer className={classes.container}>
        <Table stickyHeader aria-label="sticky table">
          <CustomTableHead columns={columns} order={order} orderBy={orderBy} onRequestSort={handleRequestSort}/>
          <TableBody>
            {stableSort(rows, orderBy != null && columns.filter(column => column.id === orderBy)[0].comparator 
            ? getComparator(order, orderBy, columns.filter(column => column.id === orderBy)[0].comparator)
            : getComparator(order, orderBy))
                .slice(page * rowsPerPage, page * rowsPerPage + rowsPerPage)
                .map((row) => {
              return (
                <TableRow hover role="checkbox" tabIndex={-1} key={row.id} onClick={(event) => handleClick(event, row.id)}>
                  {columns.map((column) => {
                    const displayedItem = row[column.id].displayed;
                    return (
                      <TableCell key={column.id} align={column.align}>
                        {column.format ? column.format(displayedItem) : displayedItem}
                      </TableCell>
                    );
                  })}
                </TableRow>
              );
            })}
          </TableBody>
        </Table>
      </TableContainer>
      <TablePagination
        rowsPerPageOptions={[10, 25, 100]}
        component="div"
        count={rows.length}
        rowsPerPage={rowsPerPage}
        page={page}
        onChangePage={handleChangePage}
        onChangeRowsPerPage={handleChangeRowsPerPage} />
    </div>
  );
}

CustomTable.propTypes = {
  columns: PropTypes.arrayOf(PropTypes.shape({id: PropTypes.string.isRequired})).isRequired,
  rows: PropTypes.array.isRequired,
  handleRowClick: PropTypes.func.isRequired,
};