import React, { createContext, useContext, useEffect, useState } from "react";
import {
  AppBar,
  Avatar,
  Button,
  IconButton,
  ListItem,
  ListItemIcon,
  ListItemText,
  makeStyles,
  Toolbar,
  Typography,
} from "@material-ui/core";
import { DesktopMac, Launch, LockOpen, Menu, PanoramaFishEye } from "@material-ui/icons";
import { MenuDrawer } from "../Menu/MenuDrawer";
import { Link } from "react-router-dom";
import { userContext } from "../../../api/SwaggerApi";

const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
    paddingBottom: theme.spacing(2),
  },
  appBar: {
    zIndex: 10000 // fixme: a hack to make the appBar stay on top of the drawer
  },
  menuButton: {
    marginRight: theme.spacing(2),
  },
  title: {
    flexGrow: 1,
  },
}));

export function ButtonAppBar({ onMenuClick, title, button, user }) {
  const classes = useStyles();

  return (
    <div className={classes.root}>
      <AppBar position="relative" className={classes.appBar}>
        <Toolbar>
          <IconButton edge="start" onClick={onMenuClick} className={classes.menuButton} color="inherit" aria-label="menu">
            <Menu />
          </IconButton>
          <Typography variant="h6" className={classes.title} noWrap>
            {title}
          </Typography>
          {button}
          <Avatar src={user.avatar} variant="circular" />
        </Toolbar>
      </AppBar>
    </div>
  );
}


const defaultButton = <Button color="inherit">Login</Button>;
const defaultTitle = "CCCE"
export const GlobalAppBarContext = createContext({
  button: defaultButton,
  title: defaultTitle,
  setButton: () => { console.log("default GlobalAppBarContext.setButton does nothing.") },
  setTitle: () => { console.log("default GlobalAppBarContext.setTitle does nothing.") }
});

function MenuListItems({ menuItems, toggleDrawer }) {
  return (
    <>
      {menuItems.map(({ text, url, icon }, index) => (
        <ListItem button key={text} component={Link} to={url} onClick={(event) => {
          toggleDrawer();
        }}>
          <ListItemIcon>{icon}</ListItemIcon>
          <ListItemText primary={text} />
        </ListItem>
      ))}
    </>
  );
}

export function UserInfo({ user }) {
  console.log(user);
  return (
    <ListItem>
      <ListItemIcon>
        <Avatar src={user.avatar} variant="rounded" />
      </ListItemIcon>
      <ListItemText primary={user.fullName} secondary={user.email} />
    </ListItem>
  );
}

export function LogoutButton({ logout }) {
  return (
      <ListItem button onClick={logout}>
        <ListItemIcon>
          <LockOpen />
        </ListItemIcon>
        <ListItemText primary="Log out" />
      </ListItem>
  );
}

export function GlobalAppBar({ children }) {
  const { user, logout } = useContext(userContext);
  const [button, setButton] = useState(defaultButton);
  const [title, setTitle] = useState(defaultTitle);
  const [value] = useState({ setButton, setTitle });
  const [drawerOpen, setDrawerOpen] = useState(false);

  const toggleDrawer = () => {
    setDrawerOpen(!drawerOpen);
  }

  const makeLink = (text, url, icon) => ({ text, url, icon });
  const menuItems = [
    makeLink("IOCs", "/iocs", <PanoramaFishEye />),
    makeLink("Hosts", "/hosts", <DesktopMac />),
    makeLink("Deployments", "/deployments", <Launch />)
  ]

  return (
    <>
      <ButtonAppBar onMenuClick={toggleDrawer} title={title} button={button} user={user} />
      <MenuDrawer header={<UserInfo user={user} />} footer={<LogoutButton logout={logout} />} open={drawerOpen} toggleDrawer={toggleDrawer}>
        <MenuListItems menuItems={menuItems} toggleDrawer={toggleDrawer} />
      </MenuDrawer>
      <GlobalAppBarContext.Provider value={value}>
        {children}
      </GlobalAppBarContext.Provider>
    </>
  );
}

export function useGlobalAppBar(initTitle, initButton) {
  const [button, setButton] = useState(initButton);
  const [title, setTitle] = useState(initTitle);
  const appBar = useContext(GlobalAppBarContext);
  useEffect(() => {
    appBar.setButton(button);
    appBar.setTitle(title);
  }, [appBar, title, button]);
  return { setButton, setTitle };
}